function c = subsasgn (c, s, b)

% @cortex/subsasgn.m
%
% Subscript handling routine for @cortex objects
%
% The user of the class does not need to know how it works.
%
% Last modified: 3 Jun 00

% 10 May 00: deltrial.m support
% 12 May 00: bug fixes
%  3 Jun 00: moving methods to private

% trap unlock and lock requests
if length (s) == 1 & s.type == '.'
   if strcmp (s.subs, 'unlock')
      c.changemode = chmode (c, checkstring (b));
      return
   elseif strcmp (s.subs, 'lock')
      c.changemode = chmode (c, ~checkstring (b));
      return
   end
end

% this is not a lock/unlock request.
% is the object locked?

if c.changemode < 2
   disp ('@CORTEX OBJECT IS LOCKED');
   disp ('** NO CHANGES APPLIED **');
   return
end

% object can be changed. proceed.
% how may levels of referencing?
switch length (s)
case 1
   % one referencing level.
   switch s.type
   case '.'
      switch lower (s.subs)
      case 'deltrial'
         c = deltrial (c, b);
      case 'name'
         c = changename (c, b);
      otherwise
         error ('invalid action');
      end
   otherwise
      error ('invalid subreferencing');
   end
   
case 2
   % two referencing levels.
   if s(1).type == '.' & s(2).type == '()'
      switch s(1).subs
      case 'changeheadvalue'
         c = changeheadvalue (c, s(2).subs{1}, s(2).subs{2}, b);
      otherwise
         error ('invalid action');
      end
   else
      error ('invalid subreferencing');
   end
   
otherwise
   error ('too many levels of subreferencing');
end   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function f2 = checkstring (sss)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isa (sss, 'char')
   if strcmp (lower(sss), 'true')
      f2 = 1;
   elseif strcmp (lower(sss), 'false')
      f2 = 0;
   else
      error ('invalid action');
   end
else
   if sss
      f2 = 1;
   else
      f2 = 0;
   end
end
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function cm = chmode (c, logico)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if logico
   switch c.changemode
   case {0, 1}
      cm = c.changemode + 2;
      disp ('**********************************');
      disp ('@CORTEX OBJECT CAN BE MODIFIED NOW');
      disp ('**********************************');
   case {2, 3}
      disp ('@cortex object is already unlocked');
      cm = c.changemode;
   otherwise
      error ('internal error. tell giuseppe!');
   end
else
   switch c.changemode
   case {0, 1}
      disp ('@cortex object is already locked');
      cm = c.changemode;
   case {2, 3}
      cm = c.changemode - 2;
      disp ('************************');
      disp ('@CORTEX OBJECT IS LOCKED');
      disp ('************************');
   otherwise
      error ('internal error. tell giuseppe!');
   end
end
return