function [ts, cl] = trialspikes (c, tr)

% @CORTEX/TRIALSPIKES   returns all spikes from a single trial
%
% [TIMES,CLUST] = TRIALSPIKES(COBJ,TRIAL)
%    COBJ:  a @cortex object
%    TRIAL: a single trial from COBJ
%    TIMES: array of time stamps in ms for all recorded spikes
%    CLUST: array of cluster codes corresponding to the time stamps
%
% Last modified: 22 Oct 04

% 20 sep 99: fix for spikeless trials.
% 22 oct 04: major rewrite; now calling TRIALCLUSTER

clus = [];
time = [];

for cluster = c.cluster_bank
   spi = trialcluster (c, cluster, tr);
   if ~isempty (spi)
      time = [time; spi];
      tc = repmat (cluster, length(time), 1);
      clus = [clus; tc];
   end
end

[ts so] = sort (time);
cl = clus(so);