function k = addfactor (k, label, levels, keys)

%@KEY/ADDFACTOR defines and stores a factor in a @spix object
%
%    K = ADDFACTOR(K,LABEL,LEVELS,KEYS) stores in the K @key object criteria to
%    perform ANOVAs. A factor has a LABEL (a string, e.g., 'color'),
%    two or more LEVELS (a cell array of strings, e.g., {'red' 'blue'
%    'green' 'yellow'}), and key sets that are used by CORTEX/ANOVAN to
%    select the appropriate spike trains (a cell array of key #s,
%    e.g., {1:4, 5:8, 9:12, 13:16}.
%
%    Note that the keys belonging to each of the levels are passed to
%    ADDFACTOR in each of the cells in KEYS, such that:
%    length(LEVELS)==length(KEYS).
%    
%Last modified 9 Feb 05

% 09 Feb 05: function created based on SPIX/ADDFACTOR

error (nargchk (4, 4, nargin));
errifnotstring (label, 'LABEL');
if ~iscell(levels) | ~iscell(keys) | length(levels) ~= length(keys)
   error ('LEVELS, KEYS must be cell arrays of equal length');
end

fi = k.f.n + 1;
k.f.factors{fi} = label;
k.f.levels{length(k.size), fi}=[];
for le = 1:length(levels)
   ks = keys{le};
   for ke = ks
      k.f.levels {ke, fi} = levels {le};
   end
end
k.f.n = fi;