function k = subsasgn (k, s, b)

% @key/subsasgn.m
%
% Subscript handling routine for @key class.
% The user of the class does not need to know how it works.
%
% Last modified: 10 Nov 04

%  8 Apr 02: last unnoted changes
% 12 Jan 03: minifield replaces try blocks
%            findex support added
% 10 Nov 04: cosmetics

switch length (s)
case 1
   if s.type == '.'
      if sum(k.size) > 1
         ss = sprintf ('there is more than one key in object\ntry: kobj(key).field');
         error (ss);
      end
      ind = 1;
      kid = 1;
      field = minifield (s.subs);
   else
      error ('bad syntax');
   end
case 2
   if s(1).type == '()' & s(2).type == '.'
      % process calls of type
      % kobj(key,subkey).field=value;
      field = minifield (s(2).subs);
      kid = findex (k, s(1).subs{1});
      base = sum (k.size (1:kid-1)) + 1;
      
      if size (s(1).subs) == [1 1]
         % whole key indexing:
         % kobj(key).field=value
         if k.size(kid)>1 & ~strcmp (field, 'lab')
            ss = sprintf ('this is a complex key\ntry: kobj(key,subkey).field=value');
            error (ss);
         end
         ind = base;
      elseif size (s(1).subs) == [1 2]
         % key, subkey indexing
         % kobj(key,subkey).field=value
         
         if k.size (kid)==1
            warning ('this is not a complex key; second index ignored');
            ind = base;
         else
            subk = s(1).subs{2};
            if subk < 1 | subk > k.size (kid)
               error ('the requested subkey does not exist');
            end
            ind = base + subk - 1;
         end
      else
         error ('wrong number of indices in @key object');
      end
   else
      error ('bad syntax');
   end
otherwise
   error ('bad syntax');
end

% now that all subscript checking is done,
% try to assign the value to a matching field
switch field
case 'lab'
   k.label{kid} = b;
case 'sta'
   k.st(ind).start = b;
case 'fin'
   k.st(ind).finish = b;
case 'cue'
   k.st(ind).cues = b;
case 'con'
   k.st(ind).conds = b;
case 'blo'
   k.st(ind).blocks = b;
case 'tri'
   k.st(ind).trials = b;
case 'res'
   k.st(ind).resps = b;
case 'typ'
   k.st(ind).type = b;
case 'exp'
   warning ('Use of dot-property ''exp'' is deprecated. Please use one of:')
   disp ('- ''typ'' if you want to set the trial type, aka the ''expected response''')
   disp ('- ''giv'' if you want to set the given response, aka the ''response''')
   k.st(ind).given = b;
case 'giv'
   k.st(ind).given = b;
case 'rep'
   k.st(ind).rept = b;
case 'rel'
   k.st(ind).rel = b;
otherwise
   error ('field does not exist');
end