function a = subsref (k, s)

% @key/subsref.m
%
% Subscript handling routine for the @key class.
% The user does not need to know how it works.
%
% Last modified: 20 Nov 04

% 12 Apr 02: last unnoted changes
%  1 Dec 02: allow k(label).field indexing
% 12 Jan 03: minifield replaces try blocks
%  1 Aug 03: k.lab returns all labels
% 20 Nov 04: expect replaced by given; rel added

% check indices: (key, subkey)
% either 1 or 2 indices, both must be scalar
% also define kid and sid for later use
if s(1).type == '()'
   kid = findex (k, s(1).subs{1});
   ind = sum (k.size (1:kid-1)) + 1;
   if size (s(1).subs) == [1 1]
      sid = 1;
   elseif size (s(1).subs) == [1 2]
      % subkey requested
      sid = s(1).subs {2};
      if length (sid) > 1
         error ('indices into a @key object must be scalar');
      end
      if sid < 1 | sid > k.size(kid)
         error ('the requested subkey does not exist');
      end
   else
      error ('wrong number of indices in @key object');
   end
end

% how many subscripting levels?
switch length (s)
   case 1
      % only one level, but what kind? '.', '()' or '{}'?
      switch s.type
         case '.'
            field = minifield (s.subs);
            switch field
               case 'pri'
                  % if k.print then call print(k)
                  print (k);
                  a=[];
                  return
               case 'siz'
                  % return the number of subkeys per key
                  a = k.size;
                  return
               case 'len'
                  % return the number of keys
                  a = length (k.size);
                  return
               case 'lab'
                  % return all key labels
                  a = k.label;
                  return
               otherwise
                  if length(k.size) > 1
                     % if k contains more than one key then abort here
                     error ('this object contains more than one key. Try: kobj(n).field');
                  else
                     ind = 1;
                     kid = 1;
                  end
            end
         case '()'
            % the following returns one simple or complex key
            % which can then be passed to functions such as spikematrix()
            if  length (s(1).subs) == 1
               % whole key requested
               lind = ind + k.size (kid) - 1;
            else
               % subkey only
               ind = ind + sid - 1;
               lind = 0;
            end
            % the position of the first subkey in the structure
            % create a new @key object using the first subkey
            a = key ( ...
               k.label{kid}, ...
               k.st(ind).start, ...
               k.st(ind).finish, ...
               k.st(ind).cues, ...
               k.st(ind).conds, ...
               k.st(ind).blocks, ...
               k.st(ind).resps, ...
               k.st(ind).trials, ...
               k.st(ind).type, ...
               k.st(ind).given, ...
               k.st(ind).rept, ...
               k.st(ind).rel ...
               );
            % append each of the following subkeys
            for ik = ind+1:lind
               t = key ( ...
                  k.label{kid}, ...
                  k.st(ik).start, ...
                  k.st(ik).finish, ...
                  k.st(ik).cues, ...
                  k.st(ik).conds, ...
                  k.st(ik).blocks, ...
                  k.st(ik).resps, ...
                  k.st(ik).trials, ...
                  k.st(ik).type, ...
                  k.st(ik).given, ...
                  k.st(ik).rept, ...
                  k.st(ik).rel ...
                  );
               
               a = plus (a, t);
            end
            return
         otherwise
            error ('{} or [] not allowed with @key objects');
      end
   case 2
      % if two subscripting levels are used, the two valid forms are:
      % value = kobj(key).field
      % value = kobj(key, subkey).field
      if s(1).type == '()' & s(2).type == '.'
         field = minifield (s(2).subs);
         % process indices
         if length (s(1).subs) == 1
            if strcmp (field, 'siz')
               a = k.size(kid);
               return;
            end
            if k.size (kid) > 1 & ~strcmp (field, 'lab')
               warning ('this is a complex key; returning value for 1st subkey only');
            end
         else
            ind = ind + sid - 1;
         end
      else
         error ('bad syntax: try kobj(n).field');
      end
   otherwise
      error ('bad syntax: too many subscripting levels');
end

% if the field exists, return the requested value
switch field
   case 'pri'
      for i=ind
         disp (struct(k.st(i)))
      end
      a = [];
   case 'siz'
      a = k.size(kid);
   case 'lab'
      a = k.label{kid};
   case 'sta'
      a = k.st(ind).start;
   case 'fin'
      a = k.st(ind).finish;
   case 'cue'
      a = k.st(ind).cues;
   case 'con'
      a = k.st(ind).conds;
   case 'blo'
      a = k.st(ind).blocks;
   case 'res'
      a = k.st(ind).resps;
   case 'tri'
      a = k.st(ind).trials;
   case 'typ'
      a = k.st(ind).type;
   case 'exp'
      warning ('Use of dot-property ''exp'' is deprecated. Please use one of:')
      disp ('- ''typ'' if you want to retrieve the trial type, aka the ''expected response''')
      disp ('- ''giv'' if you want to retrieve the given response, aka the ''response''')
      a = k.st(ind).given;
   case 'giv'
      a = k.st(ind).given;
   case 'rep'
      a = k.st(ind).rept;
   case 'rel'
      a = k.st(ind).rel;
   otherwise
      error ('bad field name');
end
