function a = subsref (r, s)

% @rules/subsref.m
%
% Subscript handling routine for @rules class.
%
% The user of the class does not need to know how it works.
%
% Last modified: 25 Jun 99

% how many subscripting levels?
switch length (s)
case 1
   % only one level, but what kind? '.', '()' or '{}'?
   switch s.type
   case '.'
      if strcmp (lower(s.subs(1:3)), 'pri')
         % if r.print then call print(r)
         print (r);
         a=[];
         return
      elseif strcmp (lower (s.subs(1:3)), 'siz')
         % if r.size then return the number of rules per group
         a = r.size;
         return
      elseif sum(r.size) > 1
         % if r contains more than one rule then abort here
         error ('there is more than one rule. Use robj(n).field');
      else
         field = s.subs;
         ind   = 1;
      end
      
   case '()'
      % the following returns the structure of one rule
      % which can then be passed to functions such as getspikearray()
      
      % there are two ways of indexing the request for the rule:
      % rstruct = robj (group, rule);
      % rstruct = robj (rule);
      % here they are:
      if size (s.subs) == [1 2]
         % in this case the user asks for robj(group, rule)...
         group = s(1).subs{1};
         rule  = s(1).subs{2};
         if group < 1 | group > length (r.size)
            error ('invalid group number');
         end
         if rule < 1 | rule > r.size (group)
            error ('invalid rule number for this group');
         end
         % find the actual rule number
         ar = sum (r.size (1:group-1)) + rule;
         a = rules (...
            r.st(ar).label, ...
            r.st(ar).start, ...
            r.st(ar).finish, ...
            r.st(ar).cues, ...
            r.st(ar).conds, ...
            r.st(ar).blocks, ...
            r.st(ar).resps, ...
            r.st(ar).trials, ...
            r.st(ar).type, ...
            r.st(ar).expect);
         return
      elseif size(s.subs) == [1 1]
         % ...while in this case the rule-th rule is requested,
         % independent of how many groups there are
         rule = s.subs{1};
         if rule < 1 | rule > sum(r.size)
            error ('invalid rule number');
         end
         a = rules (...
            r.st(rule).label, ...
            r.st(rule).start, ...
            r.st(rule).finish, ...
            r.st(rule).cues, ...
            r.st(rule).conds, ...
            r.st(rule).blocks, ...
            r.st(rule).resps, ... 
            r.st(rule).trials, ...
            r.st(rule).type, ...
            r.st(rule).expect);
         return
      else
         error ('invalid number of dimensions');
      end
   otherwise
      % braces are not allowed
      error ('cant use {} here');
   end
case 2
   % if two subscripting levels are used, the only valid forms are:
   % value = robj (group, rule).field
   % value = robj (rule).field
   
   if s(1).type == '()' & s(2).type == '.'
      field = s(2).subs;
      
      if size (s(1).subs) == [1 2]
         % the user requested a value using the (group, rule)
         % indexing syntax
         group = s(1).subs{1};
         rule  = s(1).subs{2};
         % check for bad indices
         if group < 1 | group > length (r.size)
            error ('invalid group number');
         end
         if rule < 1 | rule > r.size (group)
            error ('invalid rule number for this group');
         end
         if group == 1
            ind = rule;
         else
            ind = sum (r.size (1:group-1)) + rule;
         end
      elseif size(s(1).subs) == [1 1]
         % the user requested a value using the (rule-th)
         % indexing syntax
         ind = s(1).subs{1};
         % check for bad indices
         if ind < 1 | ind > sum(r.size)
            error ('invalid rule number');
         end
      else
         error ('invalid number of dimensions');
      end
   else
      error ('bad syntax. Use robj(n).field');
   end
otherwise
   error ('bad syntax. Use robj(n).field');
end

% if the field exists, return the requested value
if length (field) < 3
   error ('field name is too short');
end

switch lower(field(1:3))
case 'pri'
   for i=ind
      disp (struct(r.st(i)))
   end
   a = [];
case 'siz'
   a = r.size;
case 'lab'
   a = r.st(ind).label;
case 'sta'
   a = r.st(ind).start;
case 'fin'
   a = r.st(ind).finish;
case 'cue'
   a = r.st(ind).cues;
case 'con'
   a = r.st(ind).conds;
case 'blo'
   a = r.st(ind).blocks;
case 'res'
   a = r.st(ind).resps;
case 'tri'
   a = r.st(ind).trials;
case 'typ'
   a = r.st(ind).type;
case 'exp'
   a = r.st(ind).expect;
otherwise
   error ('field does not exist');
end
