function sp = addunits (sp, c, u, n, d)

%SPIX/ADDUNITS  adds units to a @spix
%
%   SP = ADDUNITS(SP,C) adds to SP all units found in the C @cortex object.
%   Each unit is labeled with the name of the source object, a dot, and the
%   cluster number.
%
%   SP = ADDUNITS(SP,C,U) adds to SP only the units specified in the vector
%   U among those contained in C.
%
%   SP = ADDUNITS(..., N) the string N is used as a label, or unit's
%   nickname. If more than one unit is specified, then a dot followed by
%   the cluster number are automatically appended to N.
%
%   SP = ADDUNITS(..., N, D) the string D is stored as a recording date.
%
%Last modified: 8 Aug 03


% Not implemented yet:
%   SP = ADDUNITS(SP, F) where F is a string containing path and name of a 
%   text file, adds to SP all units specified in the file.

error (nargchk (2, 5, nargin));
defu=false;
da='';
if isa (c, 'char')
   error ('Sorry, syntax not implemented yet');
   % [list, rec, fie, emp] = listfromfile (c);
elseif isa (c, 'cortex')
   uni = c.bank;
   if nargin > 2
      if isnumeric (u)
         defu = true;
         uni = u;
      elseif ischar (u)
         lab = u;
      else
         error ('bad argument list');
      end
   end
   if nargin > 3
      if ~isa (n, 'char'), error ('bad argument list'); end
      if defu, lab = n; else da = n; end
   end
   if nargin > 4
      if ~isa (d, 'char'), error ('bad argument list'); end
      if defu, da=d; end
   end
else
   error ('bad argument list');
end

% process requested units
for cluster = uni
   tu = sp.nunits + 1;
   % make unit's label
   if exist ('lab', 'var')
      label = [lab '.' num2str(cluster)];
   else
      label = [c.name '.' num2str(cluster)];
   end
   % check that label does not already exist
   if lbl2idx (sp.info, label)
      error (['a unit with label ' label ' already exists in the spix']);
   else
      sp.info(tu).label = label;
   end
   % create spm for current unit
   spm = [];
   for key = 1:sp.k.len
      [spt, trials] = spikematrix (c, cluster, sp.k(key));
      nt = uint16 (repmat (key, size (spt, 1), 1));
      sp.key  = [sp.key; nt];
      spm = [spm; double(spt)];
      sp.trial = [sp.trial; trials];
   end
   % append new data to spix
   sp.spix = [sp.spix; sparse(spm)];
   sp.nunits = tu;
   tn = uint16 (repmat (tu, size (spm,1), 1));
   sp.unit = [sp.unit; tn];
   sp.yest = [sp.yest; logical(ones(size(spm,1), 1))];
   sp.inclunits = [sp.inclunits, tu];
   sp.info(tu).source = c.name;
   sp.info(tu).date = da;
   sp.info(tu).extra = '';
   disp (['adding unit #' num2str(tu) ' (' sp.info(tu).label ') with ' num2str(size(spm,1)) ' trials']);
end
