function as = annsetup (factors)

%@ANNSETUP/ANNSETUP   the @annsetup object class provides a handy way to
%   prepare data for input into Matlab Stats Toolbox's ANOVAN function.
%   ANOVAN performs n-way anovas and requires properly formatted data and
%   grouping matrices. If your raw data is available in vectors of
%   observations associated with known levels for each of the factors, then
%   @annsetup will save you a few lines of clunky code.
%
%   Constructor's syntax: AS = ANNSETUP(N)
%   creates and returns an  @annsetup object.
%   N can be either the number of factors in the analysis to be performed,
%   or a cell array containing the names of each factor (recommended).
%
%Last modified: 31 Jan 06

% Last unnoted changes: 02 AUg 03
% 03 Aug 05: factor labeling added to constructor syntax
% 31 Jan 06: verbosity added

as.D = [];
if isnumscalar (factors)
   as.N = factors;
   as.F = {};
elseif iscell (factors)
   as.N = length(factors);
   as.F = factors;
else
   error ('wrong argument type');
end
as.G = repmat ({[]}, as.N, 1);
as.verbose = false;
as = class (as, 'annsetup');

disp ([num2str(as.N) '-way ANOVA with factors:']);
if isempty (as.F)
   disp ('unspecified');
else
   disp (as.F);
end
disp ('successfully set up.');