function [list, records, fields, empty] = listfromfile (filename)

%LISTFROMFILE
%
%  Reads strings and numbers from a text file into a cell array.
%  values in the same row should be separated by whitespace;
%  records should be separated by newlines.
%
%  Skips lines that begin with the character %
%
%  Typically used to load a list of file names and other parameters
%  for subsequent processing.
%
%  [LIST, RECORDS, FIELDS, EMPTY] = LISTFROMFILE (FILENAME)
%  
%  FILENAME: a valide path\filename pointing to a text file
%  LIST:     a cell array; each cell contains a value read from
%            the input file
%  RECORDS:  the number of lines successfully processed
%  FIELDS:   the number of fields in the database
%  EMPTY:    false if all cells contain values,
%            true if at least one cell is empty
%
%For example, if the file 'test.txt' contains the following lines:
%
% one [1; 2; 3]
% two   []   33
% three 3:22  omega
%
%...then...
%
%  list  = listfromfile('test.txt')
%
% list = 
%
%    'one'      [3x1  double]         []
%    'two'                 []    [   33]
%    'three'    [1x20 double]    'omega'
%
%Last modified: 15 Sep 04 --Giuseppe Bertini

% 31 aug 99: last unnoted changes
%  8 aug 02: recognizes [], allows whitespace within;
%          : 'empty' argout added
% 15 sep 04: fixed a bug causing the last line of a source file to be ignored

f=openfile(filename, 'r');
list = [];
RowIdx=0;

while ~feof(f)
   li = fgetl (f);
   %%!!   if feof(f), break, end %% BUG!
   linea = trimwhite (li);
   if isempty(linea) | linea(1)=='%', continue, end
   RowIdx = RowIdx + 1;
   ColIdx=0;
   while 1
      if linea(1)=='['
         tf = find (linea == ']');
         if isempty (tf)
            error ('missing '']'' in field')
         end
         tok = linea (1:min(tf));
         linea = linea (min(tf+1):end);
      elseif linea(1)=='{'
         tf = find (linea == '}');
         if isempty (tf)
            error ('missing ''}'' in field')
         end
         tok = linea (1:min(tf));
         linea = linea (min(tf+1):end);
      else
         [tok, linea] = strtok (linea);
      end
      
      ColIdx = ColIdx + 1;
      try
         list {RowIdx, ColIdx} = eval(tok);
      catch
         list {RowIdx, ColIdx} = tok;
      end
      linea = trimwhite(linea);
      if isempty(linea) | linea(1)=='%', break, end
   end
end
fclose (f);
[records, fields] = size (list);

if nargout==4
   empty=0;
   for row=1:RowIdx
      for col=1:ColIdx
         if isempty(list{row, col})
            empty=1;
            return;
         end
      end
   end
end